<?php

/**
 * @file
 * Pages for User Pic Kit.
 */

/**
 * Implements hook_form_BASE_FORM_ID_alter().
 *
 * Hook into user_profile_form().
 *
 * @see hook_form_BASE_FORM_ID_alter()
 */
function userpickit_form_user_profile_form_alter(&$form, &$form_state, $form_id) {
  drupal_add_css(drupal_get_path('module', 'userpickit') . '/css/userpickit.css');

  $picture_types = userpickit_types_all();
  $form['#submit'][] = 'userpickit_form_user_submit';
  $account = $form['#user'];

  $pictures = array();
  $messages = array();

  $options_active = array();
  $options_inactive = array();

  foreach ($picture_types as $picture_type => $info) {
    if (user_access('userpickit use ' . $picture_type . ' type', $account)) {
      $cache = userpickit_cache_get($picture_type, $account->uid);

      $option = $info['title'];

      // Image
      $style = _userpickit_picture_type_ui_style();

      $path = userpickit_picture_path_cache($picture_type, $account);
      $messages[$picture_type] = !empty($cache->message) ? t($cache->message, $cache->message_variables) : '';

      $default_image = userpickit_picture_type_path_default($picture_type, $account);
      if ($path->fid || $path->uri || $default_image) {
        $picture_uri = userpickit_picture_uri($picture_type, $account);
        $pictures[$picture_type] = userpickit_render_picture($account, $picture_uri, $style);

        $options_active[$picture_type] = $info['title'];
      }
      else {
        $pictures[$picture_type] = $default_image;
        $options_inactive[$picture_type] = $info['title'];
      }
    }
  }

  $form['picture']['picture_type'] = array(
    '#title' => t('Picture'),
    '#description' => t('Choose a picture to associate with your account.'),
    '#type' => 'radios',
    '#theme' => 'userpickit_picture_configuration_active',
    '#options' => $options_active,
    '#default_value' => userpickit_picture_type_user_get($account),
    '#weight' => -20,
    '#access' => !empty($options_active),
    '#userpickit_pictures' => $pictures,
    '#userpickit_messages' => $messages,
  );

  $form['picture']['picture_type_disabled'] = array(
    '#title' => t('Inactive picture types'),
    '#theme' => 'userpickit_picture_configuration_inactive',
    '#items' => $options_inactive,
    '#weight' => -15,
    '#access' => count($options_inactive),
    '#userpickit_pictures' => $pictures,
    '#userpickit_messages' => $messages,
  );

  $form['picture']['#title'] = t('Picture');
  $form['picture']['#type'] = 'fieldset';

  $form['picture']['picture_current']['#access'] = FALSE;
  $form['picture']['picture_delete']['#access'] = FALSE;

  $form['picture']['userpickit_user_delete'] = array(
    '#title' => t('Delete uploaded picture'),
    '#type' => 'checkbox',
    '#access' => userpickit_user_picture_upload_get($account->uid),
    '#description' => t('Check this box to delete your uploaded picture.'),
    '#weight' => 20,
  );
}

/**
 * Form submission callback.
 * 
 * @see userpickit_form_user_profile_form_alter()
 */
function userpickit_form_user_submit($form, $form_state) {
  $account = $form['#user'];

  if ($form_state['values']['userpickit_user_delete']) {
    // Minor quirk if 'user' is a users selected picture type:
    // Picture will not get removed until a new picture type is selected, 
    // due to file_usage of Drupal core.
    // Do not force delete as the file may be managed by other modules.
    if (userpickit_picture_type_user_get($account) != 'user') { 
      userpickit_user_picture_upload_set($account->uid, FALSE);
    }
    else {
      drupal_set_message(t('Please change your picture preference to something other than your uploaded picture before attempting to delete your uploaded picture.'), 'warning');
    }
  }

  userpickit_user_picture_reset($account->uid);
}

/**
 * Returns HTML for the user picture configuration form.
 */
function theme_userpickit_picture_configuration_active(&$variables) {
  $form = &$variables['form'];
  $result = '<div class="picture-configuration">';

  // Loop through each rendered option element
  foreach (element_children($form) as $picture_type) {
    $result .= '<div class="picture-type active">';

    // Picture
    $result .= '<label for="' . $form[$picture_type]['#id'] . '">';
    $result .= $form['#userpickit_pictures'][$picture_type];
    $result .= '</label>';

    // Title
    $result .= '<h3>' . render($form[$picture_type]) . '</h3>';

    // Description
    $result .= '<div class="picture-type-description">';
    $result .= $form['#userpickit_messages'][$picture_type];
    $result .= '</div>';

    $result .= '</div>';
  }

  $result .= '</div>';
  return $result;
}

/**
 * Returns HTML for disabled picture types on picture configuration form.
 */
function theme_userpickit_picture_configuration_inactive(&$variables) {
  $form = &$variables['form'];
  $items = array();
  $result = '<div class="picture-configuration">';

  foreach ($form['#items'] as $picture_type => $title) {
    $item = '<div class="picture-type inactive">';
    $item .= '<div class="title">' . $title . '</div>';
    $item .= '<div class="message">' . $form['#userpickit_messages'][$picture_type] . '</div>';
    $item .= '</div>';
    $items[$picture_type] = $item;
  }

  $result .= theme('item_list', array(
    'title' => t('Inactive Pictures'), 
    'items' => $items, 
  ));

  $result .= '</div>';
  return $result;
}

/**
 * Saves data from user_profile_form.
 *
 * @see hook_user_presave()
 * @see userpickit_form_user_profile_form_alter()
 */
function _userpickit_user_presave_admin(&$edit, $account, $category) {
  if (isset($edit['picture_type'])) {
    $edit['data']['userpickit_type'] = $edit['picture_type'];
  }
}

/**
 * Global settings form for User Pic Kit.
 */
function userpickit_settings_form($form, $form_state) {
  /**
   * Global settings:
   *

       * Cache time for inactive pictures
   * 
   * Per picture type settings:
   * 
   * Caching - Allow admin to choose whether an picture type should be downloaded
   * and cached locally.
   *    KEEP_REMOTE : link directly.
   *    CACHE_LOCAL
   *   
   */
  $options = array();
  $picture_types = userpickit_types_all();
  foreach ($picture_types as $picture_type => $info) {
    $options[$picture_type] = $info['title'];
  }

  $form['picture_type_default'] = array(
    '#title' => t('Default picture type'),
    '#description' => t('Picture type to use when a user is anonymous, or has not set a preference.'),
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => variable_get('userpickit_default', '')
  );

  if (module_exists('image')) {
    $form['picture_type_ui_style'] = array(
      '#type' => 'select',
      '#title' => t('Picture list display style'),
      '#options' => image_style_options(TRUE),
      '#default_value' => _userpickit_picture_type_ui_style(),
      '#description' => t('The style of image to use on the picture configuration form. Styles may be configured in the <a href="!url">Image styles</a> administration area.', array('!url' => url('admin/config/media/image-styles'))),
    );
  }

  // @see userpickit_user_picture_reset()
  $form['behavior_fail_load'] = array(
    '#title' => t('New picture error behavior'),
    '#description' => t('Behavior to use when a users new picture cannot be loaded.'),
    '#type' => 'radios',
    '#options' => array(
      USERPICKIT_BEHAVIOR_FAIL_LOAD_NEW_KEEP_LAST => t('Use last picture'),
      USERPICKIT_BEHAVIOR_FAIL_LOAD_NEW_ERASE_LAST => t('Erase last picture, reset to default')
    ),
    '#default_value' => userpickit_behavior_fail_load_get(),
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  
  return $form;
}

/**
 * Form submission callback
 *
 * @see userpickit_settings_form()
 */
function userpickit_settings_form_submit($form, &$form_state) {
  userpickit_behavior_fail_load_set($form_state['values']['behavior_fail_load']);

  variable_set('userpickit_default', $form_state['values']['picture_type_default']);
  variable_set('userpickit_picture_type_configuration_style', $form_state['values']['picture_type_ui_style']);
}

/**
 * Get current image style for picture configuration form.
 */
function _userpickit_picture_type_ui_style() {
  return variable_get(
    'userpickit_picture_type_configuration_style',
    variable_get('user_picture_style', USERPICKIT_IMAGE_STYLE_PICTURE_CONFIGURATION)
  );
}